// Android stuff

// @todo migrate menu setup in here?

// @Override
// navigator.lang always returns 'en' on Android
// use the Globalization plugin to request the proper value
l10n.navigatorLang = function(success) {
	var lang = navigator.language;

	var glob = new Globalization;
	glob.getLocaleName(function(result) {
		lang = result.value.toLowerCase().replace('_', '-');
		//console.log('globalization gave: ' + lang);
		success(lang);
	}, function(err) {
		//console.log('globalization error: ' + err);
		success(null);
	});
}

function getAboutVersionString() {
	return "1.0.1";
}

function setMenuItemState(action, state, noUpdate) {
	if(state) {
		$("command[action='" + action + "']").removeAttr("disabled");
	} else {
		$("command[action='" + action + "']").attr("disabled", "disabled");
	}
	if(!noUpdate) { 
		updateMenuState();
	}
}

(function() {
	var ANDROIDCREDITS = [
		"<a href='https://github.com/phonegap/phonegap-plugins/tree/master/Android/Globalization'>PhoneGap Globalization Plugin</a>, <a href='http://www.opensource.org/licenses/MIT'>MIT License</a>",
		"<a href='https://github.com/phonegap/phonegap-plugins/tree/master/Android/Share'>PhoneGap Share Plugin</a>, <a href='http://www.opensource.org/licenses/MIT'>MIT License</a>",
		"<a href='https://github.com/m00sey/PhoneGap-Toast'>PhoneGap Toast Plugin</a>, <a href='http://www.opensource.org/licenses/MIT'>MIT License</a>",
		"<a href='https://github.com/phonegap/phonegap-plugins/tree/master/Android/SoftKeyboard'>PhoneGap SoftKeyboard Plugin</a>, <a href='http://www.opensource.org/licenses/MIT'>MIT License</a>",
		"<a href='https://github.com/phonegap/phonegap-plugins/tree/master/Android/WebIntent'>PhoneGap WebIntent Plugin</a>, <a href='http://www.opensource.org/licenses/MIT'>MIT License</a>"
	];

	window.CREDITS.push.apply(window.CREDITS, ANDROIDCREDITS);
})();

function setPageActionsState(state) {
	setMenuItemState("read-in", state, true);
	setMenuItemState("save-page", state, true);
	setMenuItemState("share-page", state, true);
	setMenuItemState("word-of-the-day", state, true);
}

chrome.addPlatformInitializer(function() {
	$('html').addClass('android');
	if (navigator.userAgent.match(/Android 2\./)) {
		// Android 2.2/2.3 doesn't do overflow:scroll
		// so we need to engage alternate styles for phone view.
		$('html').removeClass('goodscroll').addClass('badscroll');
	}

    document.addEventListener("backbutton", onBackButton, false);
    document.addEventListener("searchbutton", onSearchButton, false);

	function onBackButton() {
		chrome.goBack();
	}

	function onSearchButton() {
		//hmmm...doesn't seem to set the cursor in the input field - maybe a browser bug???
		
		$('#searchParam').focus().addClass('active');
		$('#searchParam').bind('blur', function() {
			  $('#searchParam').removeClass('active');
			  plugins.SoftKeyBoard.hide();
			  $('#searchParam').unbind('blur');
		});
		
		plugins.SoftKeyBoard.show();
		
	}

});

chrome.addPlatformInitializer(function() {
	// For first time loading
	var origLoadFirstPage = chrome.loadFirstPage;
	chrome.loadFirstPage = function() {
		plugins.webintent.getIntentData(function(args) {
			if(args.action == "android.intent.action.VIEW" && args.uri) {
				app.navigateToPage(args.uri);
			} else if(args.action == "android.intent.action.SEARCH") {
				plugins.webintent.getExtra("query", 
					function(query) {
						search.performSearch(query, false);
					}, function(err) {
						console.log("Error in search!");
					});
			} else {
				origLoadFirstPage();
			}
		});
	};

	// Used only if we switch to singleTask
	plugins.webintent.onNewIntent(function(args) {
		if(args.uri !== null) {
			app.navigateToPage(args.uri);
		}
	});
});

function selectText() {
    PhoneGap.exec(null, null, 'SelectTextPlugin', 'selectText', []);
}

function setMenuVersion(){
	//OS and device-specific modifications to the menu 
	
	if (device.platform == "Android"){
		var oVersion = parseFloat(device.version.substr(0,3)); //we only need the first 3 chars of the OS version code
		
		//remove select text on android 3.0 and newer
		if (oVersion >= 3.0){
			console.log("over Android 3.0: removing select text");
			$('#selectTextCmd').remove();

		}
		
		//removes listen-in on android 2.2 and older
		//replaces with history for now
		if (oVersion <= 2.2){
			console.log("under Android 2.2: replacing listen in");
			$('#soundCmd').replaceWith($('#historyCmd'));
		}

	}
	
}


function sharePage() {
	// @fixme if we don't have a page loaded, this menu item should be disabled...
	var title = app.getCurrentTitle(),
		url = app.getCurrentUrl().replace(/\.m\.wiktionary/, '.wiktionary');
	window.plugins.share.show(
		{
			subject: title,
			text: url
		}
	);
}

chrome.showNotification = function(text) {
	// Using PhoneGap-Toast plugin for Android's lightweight "Toast" style notifications.
	// https://github.com/m00sey/PhoneGap-Toast
	// http://developer.android.com/guide/topics/ui/notifiers/toasts.html
	window.plugins.ToastPlugin.show_short(text);
}

function updateMenuState() {
	var d = $.Deferred();

	var menu_handlers = {
		'read-in': function() { languageLinks.showAvailableLanguages(); },
		'view-history': function() { appHistory.showHistory(); } ,
		'save-page': function() { savedPages.saveCurrentPage() },
		'view-saved-pages': function() { savedPages.showSavedPages(); },
		'share-page': function() { sharePage(); },
		'go-forward': function() { chrome.goForward(); },
		'select-text': function() { selectText(); },
		'view-settings': function() { appSettings.showSettings(); },
		'listen-sound': function() { audioPlayer.createMenuArray(); },
        'word-of-the-day': function() { chrome.loadWordoftheDay(); },
	};
	$('#appMenu command').each(function() {
		var $command = $(this),
			id = $command.attr('id'),
			msg = 'menu-' + id.replace(/Cmd$/, ''),
			label = mw.message(msg).plain();
		$command.attr('label', label);
	});

	window.plugins.SimpleMenu.loadMenu($('#appMenu')[0],
									   menu_handlers,
									   function(success) {
										   console.log(success);
										   d.resolve(success);
									   },
									   function(error) {
										   console.log(error);
										   d.reject(error);
									   });
	return d;
};

network.isConnected = function()  {
	return navigator.network.connection.type == Connection.NONE ? false : true;
}

//@Override
app.setCaching = function(enabled, success) {
	console.log('setting cache to ' + enabled);
	if(enabled) {
		window.plugins.CacheMode.setCacheMode('LOAD_CACHE_ELSE_NETWORK', success);
	} else {
		window.plugins.CacheMode.setCacheMode('LOAD_DEFAULT', success);
	}
}

window.preferencesDB.addOnSet(function(id, value) {
	window.plugins.preferences.set(id, value, function(){}, function(){});
});
